/******************************************************************
*        PoLAR (Portable Library for Augmented Reality)           *
*     Copyright (C) 2015 Inria, Université de Lorraine, CNRS      *
*                                                                 *
* This library is free software; you can redistribute it and/or   *
* modify it under the terms of the GNU General Public License     *
* as published by the Free Software Foundation; either version 3  *
* of the License, or (at your option) any later version.          *
*                                                                 *
* This library is distributed in the hope that it will be useful, *
* but WITHOUT ANY WARRANTY; without even the implied warranty of  *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the    *
* GNU General Public License for more details.                    *
*                                                                 *
*                                                                 *
* You should have received a copy of the GNU General Public       *
* License along with this library.                                *
* If not, see <http://www.gnu.org/licenses/>.                     *
******************************************************************/

#include <iostream>
#include <QApplication>
#include <osg/io_utils>

#include <PoLAR/Image.h>
#include <PoLAR/Util.h>
#include <PoLAR/Object3D.h>
#include <PoLAR/AnimatedObject3D.h>
#include "MyViewer3D.h"

int main(int argc,char ** argv)
{
    //osg::setNotifyLevel(osg::DEBUG_FP);
    QApplication app(argc, argv);
    int width;
    int height;
    osg::ref_ptr<PoLAR::Image_uc> myImage;

    // Create the viewer
    MyViewer3D viewer;

    // Read the image given in parameter
        if(PoLAR::Util::fileExists("../data/notebook.jpg"))
            myImage = new PoLAR::Image_uc("../data/notebook.jpg", true, 1);
        else
        {
            std::cerr << "../data/notebook.jpg" << ": file not found" << std::endl;
            exit(0);
        }
        width = myImage->getWidth();
        height = myImage->getHeight();

        viewer.resize(width,height);

        // Add the image read as background image
        viewer.setBgImage(myImage);

        // Show it
        viewer.bgImageOn();

        // Have the image interaction activ
        viewer.startEditImageSlot();

        // Read the type of projection used
        PoLAR::Viewer::ProjectionType pt=PoLAR::Viewer2D3D::VISION;
        pt=PoLAR::Viewer::VISION;

        // Load the projection

            osg::Matrixd P = PoLAR::Util::readProjectionMatrix("../data/test.proj");
            viewer.setProjection(P, pt);

        // Load the model
        {
            if(!PoLAR::Util::fileExists("../data/untitled.obj"))
            {
                std::cerr << "Unable to load model" << std::endl;
                exit(0);
            }
            osg::ref_ptr<PoLAR::Object3D> obj = new PoLAR::Object3D("../data/untitled.obj", true, true);
            obj->setName("object");
            obj->optimize();
            
            obj->scale(0.5f);
            obj->translate(6.0f,5.0f,-0.5f);
            
            
            // Add the object3D created from loaded model to the viewer
            viewer.addObject3D(obj.get());
            viewer.addBlendDialog(obj.get());
            viewer.setTrackNode(obj.get());
            
            // load the model 2
            if(!PoLAR::Util::fileExists("../data/untitled.obj"))
            {
                std::cerr << "Unable to load model" << std::endl;
                exit(0);
            }
            osg::ref_ptr<PoLAR::Object3D> obj2 = new PoLAR::Object3D("../data/untitled.obj", true, true);
            obj2->optimize();
            
            obj2->scale(0.5f);
            obj2->translate(4.0f,5.0f,-0.5f);
            
            
            // Add the object3D created from loaded model to the viewer
            viewer.addObject3D(obj2.get());
            viewer.addBlendDialog(obj2.get());
        }
        
        // Add ground to for the shadow
        
        {
            osg::ref_ptr<PoLAR::Object3D> ground = PoLAR::Object3D::createGround();
            ground->translate(5.0f,5.0f,-0.1f);
            viewer.addObject3D(ground.get());
            
            ground->setEditOn();
            ground->setPhantomOn();
        }
        

        // create a phantom ground to receive shadows
        {
            viewer.addLightSource(-30,60,-100, true);
            osg::ref_ptr<PoLAR::Object3D> ground = PoLAR::Object3D::createGround();
	    ground->setPhantomOn();
            viewer.addObject3D(ground.get());
        }
        
        
        // create animation
        {
            osg::ref_ptr<PoLAR::AnimatedObject3D> object = new PoLAR::AnimatedObject3D("../data/untitled.osgt");
            object->translate(4.5f,6.0f, 0.0f);
            viewer.addObject3D(object.get());
            
            object->setAnimationSpeed(1.5);
            // start playing the animation
            object->play();
        }
        // Show the widget
        viewer.center();
        viewer.show();


    app.connect( &app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()) );

    return app.exec();
}
